#!/bin/sh
# SPDX-License-Identifier: MIT


# SPDX-License-Identifier: MIT




_get_backend() {
	rootfs="$(awk '$2 == "/" && $3 == "zfs" {print $1; exit 1}' /etc/mtab)"
	[ -z "$rootfs" ] && return 1

	eroot="$(zfs get encryptionroot -Ho value "$rootfs")"
	[ -z "$eroot" ] || [ "$eroot" = "-" ] && return 1


	backend="$(zfs-tpm-list -H "$eroot" | cut -f2)"
	[ -n "$backend" ]
	return
}

_install_tpm2() {
	inst_binary zfs-tpm2-load-key
  # shellcheck disable=SC2046
	inst_library $(find /usr/lib -name 'libtss2-tcti*.so*')  # TODO: there's got to be a better way™!
	command -v tpm2_dictionarylockout > /dev/null && inst_binary tpm2_dictionarylockout
}

_install_tpm1x() {
	inst_binary zfs-tpm1x-load-key
		command -v tcsd > /dev/null && {
		inst_binary tcsd; inst_binary ip; inst_binary ss

		# tcsd exits if it doesn't have tss group and user
		grep tss /etc/group  >> "${initdir:-MUST_EXIST}/etc/group"
		grep tss /etc/passwd >> "${initdir:-MUST_EXIST}/etc/passwd"

		for f in /lib/udev/rules.d/*tpm*; do
			inst_simple "$f"
		done

		if [ -e /etc/tcsd.conf ]; then
			inst_simple /etc/tcsd.conf
			chown tss:tss "${initdir:-MUST_EXIST}/etc/tcsd.conf"
			system_ps_file="$(awk -F '[[:space:]]*=[[:space:]]*' '!/^[[:space:]]*#/ && !/^$/ && $1 ~ /system_ps_file$/ {gsub(/[[:space:]]*$/, "", $2); print $2}' /etc/tcsd.conf)"
			system_ps_file="${system_ps_file:-/var/lib/tpm/system.data}"
		fi

		# tcsd can't find SRK if state not present, refuses to start if it doesn't have parent to tss dir
		mkdir -p "${initdir:-MUST_EXIST}/$(dirname "$(dirname "$system_ps_file")")"
		[ -f "$system_ps_file" ] && inst_simple "$system_ps_file"
	}

	# localhost needs to resolve at the very least
	for f in /etc/hosts /etc/resolv.conf /etc/host.conf /etc/gai.conf; do
		[ -e "$f" ] && inst_simple "$f"
	done

	if [ -e /etc/nsswitch.conf ]; then
		inst_simple /etc/nsswitch.conf
		databases="$(awk '/^group|hosts/ {for(i = 2; i <= NF; ++i) print $i}' /etc/nsswitch.conf | LC_ALL=C sort -u)"
		for db in $databases; do
			for f in /lib/*/"libnss_$db"*; do
				inst_library "$f"
			done
		done
	fi

	command -v tpm_resetdalock > /dev/null && inst_binary tpm_resetdalock
}


check() {
	command -v zfs-tpm-list > /dev/null || return 1

  # shellcheck disable=SC2154
  if [ -n "$hostonly" ]; then
		_get_backend || return

		[ "$backend" = "TPM2"   ] && command -v zfs-tpm2-load-key  > /dev/null && return 0
		[ "$backend" = "TPM1.X" ] && command -v zfs-tpm1x-load-key > /dev/null && return 0

		return 1
  fi

  return 0
}


depends() {
	echo zfs
}


installkernel() {
	instmods '=drivers/char/tpm'
}


install() {
	inst_binary zfs-tpm-list

	if [ -n "$hostonly" ]; then
		_get_backend

		[ "$backend" = "TPM2"   ] && _install_tpm2
		[ "$backend" = "TPM1.X" ] && _install_tpm1x
	else
		command -v zfs-tpm2-load-key  > /dev/null && _install_tpm2
		command -v zfs-tpm1x-load-key > /dev/null && _install_tpm1x
	fi

	inst_hook pre-mount 89 "${moddir:-}/tzpfms-load-key.sh"  # zfs installs with 90, we *must* run beforehand
}
